
#include "VKBaseData.hh"
#include "VKSimData.hh"
#include "VKSummingData.hh"
#include "VKPileupData.hh"

#include <string>
#include <unistd.h>
#include <stdio.h>
#include <getopt.h>
#include <iostream>

using namespace std;


void printUsage(char *const *argv);
void printLongUsage(char *const *argv);

struct CommandlineOptions {
    vector<string> input;
    string output;
    bool resolution;
    bool pileup;
};

CommandlineOptions parseoptions(char *const *argv, int argc);


// main program
int main( int argc, char* argv[] ){
	
    auto cml = parseoptions(argv, argc);

    // get data
    VKBaseData * sd;
	if (cml.input.size()==1) {
	    if (!cml.pileup) sd = new VKSimData(cml.input[0]);
        else             sd = new VKPileupData(cml.input[0]);
	}
	else {
	    sd = new VKSummingData(cml.input[0], cml.input[1]);	
	}
	
	// set experimental resolution
	sd -> AddEnergyResolution(cml.resolution);
	
	// create output file
	sd -> CreateOutput(cml.output);
	
	
	return 0;
}





// Parses the input arguments if given and sets default values otherwise
CommandlineOptions parseoptions(char *const *argv, int argc) {

    string optName;
    CommandlineOptions cml{};
    cml.resolution = false;
    cml.pileup = false;

    // Print usage if no arguments provided
    if (argc==1) printUsage(argv);

    int c;

    while (1)
    {
        struct option long_options[] =
                {
                        {"help",                no_argument, 0, 'h'},
                        {"output",              required_argument, 0, 'o'},
                        {"resolution",          no_argument, 0, 'r'},
                        {"pileup",              no_argument, 0, 'p'},                        
                        {0, 0, 0, 0}
                };
                
        /* getopt_long stores the option index here. */
        int option_index = 0;

        c = getopt_long (argc, argv, "h:o:",
                         long_options, &option_index);

        /* Detect the end of the options. */
        if (c == -1)
            break;

        switch (c)
        {
            case 0:
                /* If this option set a flag, do nothing else now. */
                if (long_options[option_index].flag != 0)
                    break;

                optName = long_options[option_index].name;
                if (optName == "resolution") {
                    cml.resolution = true;
                }
                else if (optName == "pileup") {
                    cml.pileup = true;
                }
                                
            case 'o':
                cml.output = optarg;
                break;

            case 'r':
                cml.resolution = true;
                break;

            case 'p':
                cml.pileup = true;
                break;
                
            case 'h':
                printLongUsage(argv);
                exit(0);

            case '?':
                /* getopt_long already printed an error message. */
                printUsage(argv);
                break;

            default:
                abort ();
        }
    }

    // Check if they provided a root file
    if (argc == optind) {
        cerr << "Missing input file!" << endl;
        printUsage(argv);
    }
    else if (argc > optind+2) {
        cerr << "Too many input files provided! (max 2)" << endl;
        printUsage(argv);    
    }
    // The input file should come last with no switch
    for (int i = optind; i < argc; i++) cml.input.push_back(argv[i]);

    // Default output file
    if (cml.output.empty()) cml.output = "output/o.root";
    
    return cml;
}


// Print usage information
void printUsage(char *const *argv) {
/*    
    cerr << "Usage " << argv[0] << " [options] -d detector_name input_file.root [more input files]" << endl;
    cerr << "For additional help run with -h" << endl;
    exit(1);
*/
}

void printLongUsage(char *const *argv) {
/*
    cerr << "Usage: "  << argv[0] << " [options] -d detector_name input_file.root" << endl;

    cerr << "Options: " << endl;
    fprintf(stderr, "  -s/--setup             AUSAlib setup file\n"
                    "                             (default = setup/setup.json)\n");
    fprintf(stderr, "  -o/--output-directory  Directory where output will be place\n"
                    "                             (default = output/)\n");
    fprintf(stderr, "  -k/--source            JSON file describing calibration source\n"
                    "                             (default = source.json)\n");
    fprintf(stderr, "  -t/--target            JSON file describing target foil. Used for energy loss.\n"
            "                             (default = none)\n");
    fprintf(stderr, "  --treename          Name of the tree in the data file\n"
            "                                     (default = h101)\n");
    fprintf(stderr, "  -b/--bits              Number of ADC bits\n"
            "                                     (default = 12)\n");
    fprintf(stderr, "  -n/--n-bins            Number histogram bins\n"
            "                                     (default = 2^#bits)\n");
    fprintf(stderr, "  -l/--low-threshold     Lower threshold\n"
            "                                     (default = 200)\n");
    fprintf(stderr, "  -d/--detector          Which detector to calibrate. Must be in setup.json   \n");
    fprintf(stderr, "  -i/--ion               Ion type used for energy loss calculations\n"
                    "                             (default = He4)\n");
    fprintf(stderr, "  -m/--material          Material type used for dead layer in energy loss calculations\n"
            "                                     (default = Silicon)\n");
    fprintf(stderr, "  -w/--null-width        Region around peak to set equal to zero.\n"
                    "                             (default = 300)\n");
    fprintf(stderr, "  -r/--range-mul         Gaussian fit is done within mean ± R*sigma.\n"
            "                                     (default = 3)\n");
    fprintf(stderr, "  --eloss                Correct for energy loss in deadlayer\n"
                    "                             (Not on by default)\n");
    fprintf(stderr, "  --no-eloss             Do not correct for energy loss in deadlayer\n"
                    "                             (This is the default)\n");
    fprintf(stderr, "  --remove-raw           Remove file with raw spectra\n");
    fprintf(stderr, "  -V/--verbose           Outputs all information from the underlying ROOT routines.\n");
    fprintf(stderr, "  --gui                  Starts interactive GUI review!\n");
    */
}

