//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
#ifndef VKLeadBlock_H
#define VKLeadBlock_H 1

#include "G4PVPlacement.hh"
#include "G4Tubs.hh"
#include "G4Material.hh"
#include "G4LogicalVolume.hh"
#include "G4Box.hh"

#include "VKDetectorConstruction.hh"
#include "VKMaterials.hh"

class VKLeadBlock : public G4PVPlacement
{
  public:

    /*
     * Construct a rectangular lead block
     * @param pRot Orientation of magnet 
     * @param tlate Position of magnet
     * @param pMotherLogical Place magnet inside this volume
     * @param pMany Boolean operations
     * @param pCopyNo Copy number
     * @param pDetConstr Pointer to main detector constructor
     */
    VKLeadBlock(G4RotationMatrix *pRot,
             const G4ThreeVector &tlate,
             G4LogicalVolume *pMother_log,
             G4bool pMany,
             G4int pCopyNo,
             const VKDetectorConstruction* pDetConstr);

  private:

    void VisAttributes();
    void SetDefaults();
    
    const VKDetectorConstruction* fDetConstr;
    VKMaterials* fMaterials;
    
    // Dimensions
    G4double fXDim, fYDim, fZDim;
    
    // Position
    G4double fXPos, fYPos, fZPos;
    
    // Geometry
    bool fCheckOverlaps;
    G4LogicalVolume* fMother_log;    
    G4Box* fLeadBlock_sol;
    G4LogicalVolume* fLeadBlock_log;
};

#endif
