//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
#include "G4VUserTrackInformation.hh"
#include "globals.hh"

#ifndef VKUserTrackInformation_h
#define VKUserTrackInformation_h 1


enum VKTrackStatus 
{ 
    active = 1, 
    hitSiPM = 2, 
    absorbed = 4, 
    boundaryAbsorbed = 8,
    inactive = 14
};

/*
 * VKTrackStatus:
 *
 * active: still being tracked
 * hitSiPM: stopped by being detected in a PMT
 * absorbed: stopped by being absorbed with G4OpAbsorbtion
 * boundaryAbsorbed: stopped by being aborbed with G4OpAbsorbtion
 * inactive: track is stopped for some reason
 *  -This is the sum of all stopped flags so can be used to remove stopped flags
 */


class VKUserTrackInformation : public G4VUserTrackInformation
{
  public:

    VKUserTrackInformation();
    virtual ~VKUserTrackInformation();

    // Set the track status (does not check validity of flags)
    // @param status Track status
    void SetTrackStatusFlags(int status){fStatus=status;}
    
    // Does a smart add of track status flags (disabling old flags that conflict)
    // If status conflicts with itself it will not be detected
    // @param status Track status
    void AddTrackStatusFlag(int status);
 
    // Get track status
    inline int GetTrackStatus() const { return fStatus; }
 
    // Increment reflections counter
    void IncReflections(){ fReflections++; }

    // Get current value of reflections counter
    G4int GetReflectionCount()const { return fReflections; }

    // Set forced drawing of trajectories
    // @param b Enable/disable
    void SetForceDrawTrajectory(G4bool b) { fForcedraw=b; }

    // Is forced drawing enabled/disabled?
    G4bool GetForceDrawTrajectory() const { return fForcedraw; }

    inline virtual void Print() const{};

  private:

    int fStatus;
    G4int fReflections;
    G4bool fForcedraw;
};

#endif
